'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved
'
' @file		testGPS.bs2
' @target	ARMmite/ARMexpress
'
' This demo program reads latitude and longitude values from a GPS receiver
' and displays them in the terminal window using the DEBUG command. The 4800
' baud serial OUTPUT from the GPS receiver is connected TO the SERIN PIN of
' the uM-FPU V3.1 chip. The sentence count and sentence type is displayed
' for each record, and the longitude and latitude values are displayed for the
' GPRMC sentence. The sentence types will vary depending on the GPS receiver
' used. The following example shows the output from a Garmin receiver:
'
'   GPSdemo1
'   uM-FPU V3.1.0
'   GPS enabled
'
'   1: GPGLL
'   2: GPBOD
'   3: GPVTG
'   4: PGRME
'   5: PGRMZ
'   6: PGRMM
'   7: HCHDG
'   8: GPRTE
'   9: GPRMC Lat = 7622.4609, Long = 7622.4609
'   (etc.)
' ...
'
' See Application Note 36 - Reading GPS Data
'
' @author Cam Thompson, Micromega Corporation
' @version
'   August 3, 2007
'	- ported to ARMmite/ARMexpress
'   March 15, 2007
'   - original version
'==============================================================================

'-------------------- uM-FPU V3 definitions -----------------------------------

#include "FPUspi.bas"

'==============================================================================
'==================== main definitions ========================================
'==============================================================================

Latitude        CON     8
Longitude       CON     9
cnt             VAR     Word            ' counter

GPS_OK          CON     $80             ' GPS sentence OK status

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "GPSdemo1"
	testMode = 0
		
	gosub Fpu_Reset						' reset the uM-FPU V3 chip
	if status <> SYNC_CHAR then			' check for synchronization
		print "uM-FPU not detected"
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		print
	endif

	if testMode then
		print "Test Mode"
	else
		print "GPS enabled"
		SPIOUT FpuCS, FpuOut, FpuClk, [F_SEROUT, 0, 5]  ' set 4800 baud
		SPIOUT FpuCS, FpuOut, FpuClk, [F_SERIN, 4]      ' enable NMEA
	endif
	
	cnt = 0
	do
		cnt = cnt + 1
		print
		print cnt;": ";
		
		if testMode then
			SPIOUT FpuCS, FpuOut, FpuClk, [STRSET, "GPRMC,140123.000,A,4415.5312,N,07622.4761,W,0.79,95.83,260407,,", 0]
			SPIOUT FpuCS, FpuOut, FpuClk, [STRFIELD, 1]
			status = GPS_OK
		else
			SPIOUT FpuCS, FpuOut, FpuClk, [F_SERIN, 6]
			gosub Fpu_ReadStatus
		endif

		if status = GPS_OK then
			gosub Print_StringSel
			
			SPIOUT FpuCS, FpuOut, FpuClk, [STRCMP, "GPRMC", 0]
			gosub Fpu_ReadStatus
			if status & STATUS_ZERO then
				SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Latitude, STRFIELD, 4, STRTOF, FSET0]
				SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Longitude, STRFIELD, 6, STRTOF, FSET0]
				SPIOUT FpuCS, FpuOut, FpuClk, [STRSET, 0]
				SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Latitude]
				print " Lat = ";
				gosub Print_Float
				SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, Longitude]
				print ", Long = ";
				gosub Print_Float
			endif

		else
			print
			print
			print "*** data error ***"
		endif
		if testMode then exit
	loop

Done:
	print
	print "Done."
	end

'------------------------------------------------------------------------------

Print_StringSel:
	gosub Fpu_Wait						' wait until uM-FPU is ready
	SPIOUT FpuCS, FpuOut, FpuClk, [READSEL]	
	gosub FPU_ReadDelay
	SPIIN FpuCS, FpuIn, FpuClk, [str$]	' read string from uM-FPU
	print str$;
	return
